//!	EOS_RealtimeScheduler.h


//--------------------------- inclusion guards ----------------------
#ifndef _EOS_RealtimeScheduler_h_
#define _EOS_RealtimeScheduler_h_


//--------------------------- include section -----------------------
#include "EOS_Timer.h"
#include "EOS_InitTask.h"
#include "EOS_IdleTask.h"
#include "EOS_DelayedTask.h"
#include "EOS_PeriodicTask.h"
#include "EOS_List.h"
#include "Mutex.h"


//--------------------------- class definition ----------------------
//! the class CEOS_RealtimeScheduler is an eventhandler (CEOS_TaskHandler)
//! and contains the functions which call 3 types of tasks:\n
//! idle tasks (will be called at the startup phase of the program
//! execution)\n
//! periodic tasks (will be called periodically in the running phase
//! of the program execution)\n
//! periodic real time tasks (will be called periodically and will
//! interrupt other running tasks (non-real-time tasks and real-time
//! tasks with lower priority)\n
//! idle tasks (will be called whenever the scheduler does not call
//! periodic tasks in the running phase of the program execution)
class CEOS_RealtimeScheduler
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! the member variables will not be initialized. The init values
    //! have to be all zeros, so the zero init can do it
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CEOS_RealtimeScheduler);


//--------------------------- public access -------------------------
public:
    // addDelayedTask -----------------------------------------------
    //! \internal
    void addDelayedTask(CEOS_DelayedTask* pDelayedTask,
                        uint8 nPriority);

    // addPeriodicTask ----------------------------------------------
    //! \internal
    void addPeriodicTask(CEOS_PeriodicTask* pPeriodicTask,
                         uint8 nPriority);

    // schedule -----------------------------------------------------
    void schedule();


//--------------------------- protected access ----------------------
protected:



//--------------------------- protected types -----------------------
protected:
    // SDelayedAndPeriodicTask --------------------------------------
    typedef struct
    {
        CEOS_DelayedTask* pTask;
        uint8 nPriority;
    } SDelayedAndPeriodicTask;


//--------------------------- protected data ------------------------
protected:
    // m_DelayedAndPeriodicTaskList ---------------------------------
    //! \internal
    CEOS_List<SDelayedAndPeriodicTask> m_DelayedAndPeriodicTaskList;

    // m_Mutex --------------------------------------------
    //! \internal
    //! avoids conflicts with the counter for the periodic tasks
    CMutex m_Mutex;

    // m_bInRTScheduler ---------------------------------------------
    volatile bool m_bInRTScheduler;

    // m_nPriority --------------------------------------------------
    volatile uint8 m_nPriority;
};


extern CEOS_RealtimeScheduler EOS_RealtimeScheduler;

//-------------------------------------------------------------------
// scheduleRealtime
template <class T_OBJECT>
void scheduleRealtime(T_OBJECT* pObject,
                      void(T_OBJECT::* pFunc)(),
                      unsigned int nPeriod,
                      unsigned int nDelayedExecution = 0,
                      uint8 nPriority = 0)
{
    CEOS_PeriodicTask* pTask = new CEOS_PeriodicTask(pObject,pFunc,nPeriod,nDelayedExecution);
    EOS_RealtimeScheduler.addPeriodicTask(pTask,nPriority);
}




#endif
