//!	EOS_DelayedTask.h


//--------------------------- inclusion guards ----------------------
#ifndef _EOS_DelayedTask_h_
#define _EOS_DelayedTask_h_


//--------------------------- include section -----------------------
#include "types.h"
#include "EOS_Task.h"
#include "EOS_Delete.h"


//--------------------------- class definition ----------------------
//! A CEOS_DelayedTask represents a task, which waits for its
//! execution. This class is usually only used by the EOS_Scheduler
//! resp. EOS_RealtimeScheduler
class CEOS_DelayedTask :
        public CEOS_Task
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! the execution of this task is delayed by nTimeout
    //! \param pFunc is the function
    //! \param nPeriod is the period used to wind up the timer when
    //! elapsed
    //! \param nTimeout is the delay for the execution
    CEOS_DelayedTask(void(*pFunc)(),
                     unsigned int nTimeout) :
        CEOS_Task(pFunc),
        m_nTimeout(nTimeout)
    {
        // Nothing to do
    }

    // constructor --------------------------------------------------
    //! \internal
    //! the execution of this task is delayed by nTimeout
    //! \param pObject is the pointer to the object holding the
    //! member function being executed
    //! \param pFunc is the member function
    //! \param nPeriod is the period used to wind up the timer when
    //! elapsed
    //! \param nTimeout is the delay for the execution
    template <class T_OBJECT>
    CEOS_DelayedTask(T_OBJECT* pObject,
                     void(T_OBJECT::* pFunc)(),
                     unsigned int nTimeout) :
        CEOS_Task(pObject,pFunc),
        m_nTimeout(nTimeout)
    {
        // Nothing to do
    }

    NO_COPY_AND_ASSIGNMENT(CEOS_DelayedTask);


//--------------------------- public access -------------------------
public:
    // isFinished ---------------------------------------------------
    //! \internal
    //! the task is finished, when the delay time has run out
    inline bool isFinished() const
    {
        return m_nTimeout == 0;
    }

    // elapse -------------------------------------------------------
    //! \internal
    //! counts down the timeout until it reaches 0
    void elapse(unsigned int nTime)
    {
        if (m_nTimeout >= nTime)
        {
            m_nTimeout -= nTime;
        }
        else
        {
            m_nTimeout = 0;
        }
    }

    // timeout ------------------------------------------------------
    //! \internal
    //! virtual function, that calls just execute from the CEOS_Task
    //! class
    inline virtual void timeout()
    {
        execute();
    }

    // windUp -------------------------------------------------------
    //! \internal
    //! the timer of the CEOS_DelayedTask can be wind up to a new
    //! timeout value
    void windUp(unsigned int nTimeout)
    {
        m_nTimeout = nTimeout;
    }

    // getTimeout ---------------------------------------------------
    //! \internal
    //! \return the time until the task has to be executed
    inline unsigned int getTimeout() const
    {
        return m_nTimeout;
    }


//--------------------------- protected data ------------------------
protected:
    // m_nTimeout ---------------------------------------------------
    //! \internal
    //! storing the timeout until execution
    unsigned int m_nTimeout;
};


#endif
