//!	EOS_Scheduler.cpp


//--------------------------- include section -----------------------
#include "EOS_Scheduler.h"


//--------------------------- global data ---------------------------
    CEOS_Scheduler EOS_Scheduler;


//-------------------------------------------------------------------
// onInit
void onInit(void(*pFunc)(),
            uint8 nPriority)
{
    CEOS_InitTask* pTask = new CEOS_InitTask(pFunc);
    EOS_Scheduler.addInitTask(pTask,nPriority);
}


//-------------------------------------------------------------------
// onIdle
void onIdle(void(*pFunc)(),
            uint8 nPriority)
{
    CEOS_IdleTask* pTask = new CEOS_IdleTask(pFunc);
    EOS_Scheduler.addIdleTask(pTask,nPriority);
}


//-------------------------------------------------------------------
// schedule
void schedule(void(*pFunc)(),
              unsigned int nPeriod,
              unsigned int nDelayedExecution,
              uint8 nPriority)
{
    CEOS_PeriodicTask* pTask = new CEOS_PeriodicTask(pFunc,nPeriod,nDelayedExecution);
    EOS_Scheduler.addPeriodicTask(pTask,nPriority);
}


//-------------------------------------------------------------------
// onDelay
void onDelay(void(*pFunc)(),
             unsigned int nDelayedExecution,
             uint8 nPriority)
{
    CEOS_DelayedTask* pTask = new CEOS_DelayedTask(pFunc,nDelayedExecution);
    EOS_Scheduler.addDelayedTask(pTask,nPriority);
}


//-------------------------------------------------------------------
// addInitTask
void CEOS_Scheduler::addInitTask(CEOS_InitTask* pTask,
                                 uint8 nPriority)
{
    SInitTask sTask;
    sTask.pTask = pTask;
    sTask.nPriority = nPriority;

    // Sort in the item
    CEOS_List<SInitTask>::TPosition pPosition = m_InitTaskList.first();
    while (pPosition != NULL)
    {
        const SInitTask& sCheckTask = m_InitTaskList.get(pPosition);
        if (sCheckTask.nPriority < nPriority)
        {
            m_InitTaskList.insertBefore(pPosition,sTask);
            return;
        }
        pPosition = m_InitTaskList.next(pPosition);
    }
    m_InitTaskList.add(sTask);
}


//-------------------------------------------------------------------
// addIdleTask
void CEOS_Scheduler::addIdleTask(CEOS_IdleTask* pTask,
                                 uint8 nPriority)
{
    SIdleTask sTask;
    sTask.pTask = pTask;
    sTask.nPriority = nPriority;

    // Sort in the item
    CEOS_List<SIdleTask>::TPosition pPosition = m_IdleTaskList.first();
    while (pPosition != NULL)
    {
        const SIdleTask& sCheckTask = m_IdleTaskList.get(pPosition);
        if (sCheckTask.nPriority < nPriority)
        {
            m_IdleTaskList.insertBefore(pPosition,sTask);
            return;
        }
        pPosition = m_IdleTaskList.next(pPosition);
    }
    m_IdleTaskList.add(sTask);
    if (m_pIdleTaskPosition == NULL)
    {
        m_pIdleTaskPosition = m_IdleTaskList.first();
    }
}


//-------------------------------------------------------------------
// addDelayedTask
void CEOS_Scheduler::addDelayedTask(CEOS_DelayedTask* pTask,
                                    uint8 nPriority)
{
    SDelayedAndPeriodicTask sTask;
    sTask.pTask = pTask;
    sTask.nPriority = nPriority;

    // Sort in the item
    CEOS_List<SDelayedAndPeriodicTask>::TPosition pPosition = m_DelayedAndPeriodicTaskList.first();
    while (pPosition != NULL)
    {
        const SDelayedAndPeriodicTask& sCheckTask = m_DelayedAndPeriodicTaskList.get(pPosition);
        if (sCheckTask.nPriority < nPriority)
        {
            m_DelayedAndPeriodicTaskList.insertBefore(pPosition,sTask);
            return;
        }
        pPosition = m_DelayedAndPeriodicTaskList.next(pPosition);
    }
    m_DelayedAndPeriodicTaskList.add(sTask);
}


//-------------------------------------------------------------------
// addPeriodicTask
void CEOS_Scheduler::addPeriodicTask(CEOS_PeriodicTask* pTask,
                                     uint8 nPriority)
{
    // Same list DelayedAndPeriodicTask
    addDelayedTask(pTask,
                   nPriority);
}


//-------------------------------------------------------------------
// runInitTask
void CEOS_Scheduler::runInitTask()
{
    // Init tasks are run only once
    while (m_InitTaskList.first() != NULL)
    {
        CEOS_List<SInitTask>::TPosition pPosition = m_InitTaskList.first();
        const SInitTask& sTask = m_InitTaskList.get(pPosition);
        sTask.pTask->execute();
        delete sTask.pTask;
        m_InitTaskList.remove(pPosition);
    }
}


//-------------------------------------------------------------------
// runIdleTask
void CEOS_Scheduler::runIdleTask()
{
    // Run one idle task
    if (m_pIdleTaskPosition != NULL)
    {
        const SIdleTask& sTask = m_IdleTaskList.get(m_pIdleTaskPosition);
        sTask.pTask->execute();
        // Next call, next idle task
        m_pIdleTaskPosition = m_IdleTaskList.next(m_pIdleTaskPosition);
        if (m_pIdleTaskPosition == NULL)
        {
            m_pIdleTaskPosition = m_IdleTaskList.first();
        }
    }
}


//-------------------------------------------------------------------
// getTimeout
unsigned int CEOS_Scheduler::getTimeout()
{
    // Assume maximum time to wait
    unsigned int nTimeout = ~0;

    // Find the lowest time
    CEOS_List<SDelayedAndPeriodicTask>::TPosition pPosition = m_DelayedAndPeriodicTaskList.first();
    while (pPosition != NULL)
    {
        const SDelayedAndPeriodicTask& sTask = m_DelayedAndPeriodicTaskList.get(pPosition);
        unsigned int nTaskTimeout = sTask.pTask->getTimeout();
        if (nTimeout > nTaskTimeout)
        {
            nTimeout = nTaskTimeout;
        }
        pPosition = m_DelayedAndPeriodicTaskList.next(pPosition);
    }
    return nTimeout;
}


//-------------------------------------------------------------------
// runDelayedAndPeriodicTask
void CEOS_Scheduler::runDelayedAndPeriodicTask(unsigned int nTime)
{
    // Count down all tasks
    CEOS_List<SDelayedAndPeriodicTask>::TPosition pPosition = m_DelayedAndPeriodicTaskList.first();
    while (pPosition != NULL)
    {
        const SDelayedAndPeriodicTask& sTask = m_DelayedAndPeriodicTaskList.get(pPosition);
        sTask.pTask->elapse(nTime);
        pPosition = m_DelayedAndPeriodicTaskList.next(pPosition);
    }

    // Now execute the tasks
    pPosition = m_DelayedAndPeriodicTaskList.first();
    while (pPosition != NULL)
    {
        const SDelayedAndPeriodicTask& sTask = m_DelayedAndPeriodicTaskList.get(pPosition);
        if (sTask.pTask->getTimeout() == 0)
        {
            sTask.pTask->timeout();
        }
        if (sTask.pTask->isFinished())
        {
            delete sTask.pTask;
            CEOS_List<SDelayedAndPeriodicTask>::TPosition pNextPosition = m_DelayedAndPeriodicTaskList.next(pPosition);
            m_DelayedAndPeriodicTaskList.remove(pPosition);
            pPosition = pNextPosition;
        }
        else
        {
            pPosition = m_DelayedAndPeriodicTaskList.next(pPosition);
        }
    }
}


//-------------------------------------------------------------------
// schedule
void CEOS_Scheduler::schedule(unsigned int nTime)
{
    runDelayedAndPeriodicTask(nTime);
}

