//!	EOS_Scheduler.h


//--------------------------- inclusion guards ----------------------
#ifndef _EOS_Scheduler_h_
#define _EOS_Scheduler_h_


//--------------------------- include section -----------------------
#include "EOS_InitTask.h"
#include "EOS_IdleTask.h"
#include "EOS_DelayedTask.h"
#include "EOS_PeriodicTask.h"
#include "EOS_List.h"


//--------------------------- class definition ----------------------
//! the class CEOS_Scheduler holds several types of tasks and
//! executes then when necessary. The CEOS_Scheduler is used for
//! execution of sequential non-preemptive tasks. This gives the
//! advantage for no need of process synchronization
class CEOS_Scheduler
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! the member variables will not be initialized. The init values
    //! have to be all zeros, so the zero init can do it
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CEOS_Scheduler);


//--------------------------- public access -------------------------
public:
    // addInitTask --------------------------------------------------
    //! \internal
    //! adds an init task to the init task list
    //! \param pTask is the pointer to the task
    //! \param nPriority is the priority for sorting in the task
    void addInitTask(CEOS_InitTask* pTask,
                     uint8 nPriority);

    // addIdleTask --------------------------------------------------
    //! \internal
    //! adds an idle task to the idle task list
    //! \param pTask is the pointer to the task
    //! \param nPriority is the priority for sorting in the task
    void addIdleTask(CEOS_IdleTask* pTask,
                     uint8 nPriority);

    // addDelayedTask -----------------------------------------------
    //! \internal
    //! adds a delayed task to the delayed and periodic task list
    //! \param pTask is the pointer to the task
    //! \param nPriority is the priority for sorting in the task
    void addDelayedTask(CEOS_DelayedTask* pDelayedTask,
                        uint8 nPriority);

    // addPeriodicTask ----------------------------------------------
    //! \internal
    //! adds a periodic task to the delayed and periodic task list
    //! \param pTask is the pointer to the task
    //! \param nPriority is the priority for sorting in the task
    void addPeriodicTask(CEOS_PeriodicTask* pPeriodicTask,
                         uint8 nPriority);

    // runInitTask --------------------------------------------------
    //! executes all init tasks and deletes them afterwards
    void runInitTask();

    // runIdleTask --------------------------------------------------
    //! executes all idle tasks in the order how they are sorted in
    //! the list
    void runIdleTask();

    // getTimeout ---------------------------------------------------
    //! returns the time which might elapse to the next schedule
    //! \code
    //! // inside the main() function
    //! unsigned int nTimeout = EOS_Scheduler.getTimeout();
    //! sleep that number of milliseconds (*1000 for us)
    //! ussleep(nTimeout * 1000);
    //! // was sleeping nTimeout ms
    //! EOS_Scheduler(nTimeout);
    //! \endcode
    //! \return milliseconds for next schedule
    unsigned int getTimeout();

    // schedule -----------------------------------------------------
    //! \return milliseconds for next schedule
    void schedule(unsigned int nTime);


//--------------------------- protected access ----------------------
protected:
    // runDelayedAndPeriodicTask ------------------------------------
    //! \internal
    //! \return milliseconds for next schedule
    void runDelayedAndPeriodicTask(unsigned int nTime);


//--------------------------- protected types -----------------------
protected:
    // SInitTask ----------------------------------------------------
    //! \internal
    //! holding the pointer to the task and the priority
    typedef struct
    {
        CEOS_InitTask* pTask;
        uint8 nPriority;
    } SInitTask;

    // SIdleTask ----------------------------------------------------
    //! \internal
    //! holding the pointer to the task and the priority
    typedef struct
    {
        CEOS_IdleTask* pTask;
        uint8 nPriority;
    } SIdleTask;

    // SDelayedAndPeriodicTask --------------------------------------
    //! \internal
    //! holding the pointer to the task and the priority
    typedef struct
    {
        CEOS_DelayedTask* pTask;
        uint8 nPriority;
    } SDelayedAndPeriodicTask;


//--------------------------- protected data ------------------------
protected:
    // m_InitTaskList -----------------------------------------------
    //! \internal
    CEOS_List<SInitTask> m_InitTaskList;

    // m_IdleTaskList -----------------------------------------------
    //! \internal
    CEOS_List<SIdleTask> m_IdleTaskList;

    // m_pIdleTaskPosition ------------------------------------------
    //! \internal
    //! will execute only one task each call, therefore the position
    //! within the list needs to be known
    CEOS_List<SIdleTask>::TPosition m_pIdleTaskPosition;

    // m_DelayedAndPeriodicTaskList ---------------------------------
    //! \internal
    CEOS_List<SDelayedAndPeriodicTask> m_DelayedAndPeriodicTaskList;
};


extern CEOS_Scheduler EOS_Scheduler;


//###################################################################
//########################### C like function calls #################
//###################################################################

//-------------------------------------------------------------------
// onInit
//! adds an init task to the scheduler
//! \param pFunc is the pointer to a function
//! \param nPriority is the priority for execution (the higher the
//! earlier it will be called), per default 0
//! \code
//! CMyClass::CMyClass()
//! {
//!     onInit(myInitFunction,2);
//! }
//! \endcode
extern void onInit(void(*pFunc)(),
                   uint8 nPriority = 0);


//-------------------------------------------------------------------
// onIdle
//! adds an idle task to the scheduler
//! \param pFunc is the pointer to the static function
//! \param nPriority is the priority for execution (the higher the
//! earlier it will be called), per default 0
//! \code
//! CMyClass::CMyClass()
//! {
//!     onIdle(this,&CMyClass::myIdleFunction);
//! }
//! \endcode
extern void onIdle(void(*pFunc)(),
                   uint8 nPriority = 0);


//-------------------------------------------------------------------
// schedule
//! adds a periodic task to the scheduler
//! \param pFunc is the pointer to the static function
//! \param nPeriod is the period when the task shall be re-executed
//! \param nDelayedExecution is the delay when the task shall be
//! executed the first time (0 means the task shall be delayed with
//! the same time as nPeriod)
//! \param nPriority is the priority for execution (the higher the
//! earlier it will be called in case of conflicts), per default 0
//! \code
//! CMyClass::CMyClass()
//! {
//!     schedule(&myIdleFunction,100);
//! }
//! \endcode
extern void schedule(void(*pFunc)(),
                     unsigned int nPeriod,
                     unsigned int nDelayedExecution = 0,
                     uint8 nPriority = 0);


//-------------------------------------------------------------------
// onDelay
//! adds a delayed task to the scheduler
//! \param pFunc is the pointer to the member function
//! \param nDelayedExecution is the delay when the task shall be
//! executed
//! \param nPriority is the priority for execution (the higher the
//! earlier it will be called in case of conflicts), per default 0
//! \code
//! CMyClass::CMyClass()
//! {
//!     schedule(this,&CMyClass::myIdleFunction);
//! }
//! \endcode
extern void onDelay(void(*pFunc)(),
                    unsigned int nDelayedExecution,
                    uint8 nPriority = 0);


//###################################################################
//########################### C++ function calls ####################
//###################################################################

//-------------------------------------------------------------------
// onInit
//! adds an init task to the scheduler
//! \param pObject is the pointer to the object containing the member
//! function being executed
//! \param pFunc is the pointer to the member function
//! \param nPriority is the priority for execution (the higher the
//! earlier it will be called), per default 0
//! \code
//! CMyClass::CMyClass()
//! {
//!     onInit(this,&CMyClass::myInitFunction,3);
//! }
//! \endcode
template <class T_OBJECT>
void onInit(T_OBJECT* pObject,
            void(T_OBJECT::* pFunc)(),
            uint8 nPriority = 0)
{
    CEOS_InitTask* pTask = new CEOS_InitTask(pObject,pFunc);
    EOS_Scheduler.addInitTask(pTask,nPriority);
}


//-------------------------------------------------------------------
// onIdle
//! adds an idle task to the scheduler
//! \param pObject is the pointer to the object containing the member
//! function being executed
//! \param pFunc is the pointer to the member function
//! \param nPriority is the priority for execution (the higher the
//! earlier it will be called), per default 0
//! \code
//! CMyClass::CMyClass()
//! {
//!     onIdle(this,&CMyClass::myIdleFunction);
//! }
//! \endcode
template <class T_OBJECT>
void onIdle(T_OBJECT* pObject,
            void(T_OBJECT::* pFunc)(),
            uint8 nPriority = 0)
{
    CEOS_IdleTask* pTask = new CEOS_IdleTask(pObject,pFunc);
    EOS_Scheduler.addIdleTask(pTask,nPriority);
}


//-------------------------------------------------------------------
// schedule
//! adds a periodic task to the scheduler
//! \param pObject is the pointer to the object containing the member
//! function being executed
//! \param pFunc is the pointer to the member function
//! \param nPeriod is the period when the task shall be re-executed
//! \param nDelayedExecution is the delay when the task shall be
//! executed the first time (0 means the task shall be delayed with
//! the same time as nPeriod)
//! \param nPriority is the priority for execution (the higher the
//! earlier it will be called in case of conflicts), per default 0
//! \code
//! CMyClass::CMyClass()
//! {
//!     schedule(this,&CMyClass::myIdleFunction);
//! }
//! \endcode
template <class T_OBJECT>
void schedule(T_OBJECT* pObject,
              void(T_OBJECT::* pFunc)(),
              unsigned int nPeriod,
              unsigned int nDelayedExecution = 0,
              uint8 nPriority = 0)
{
    CEOS_PeriodicTask* pTask = new CEOS_PeriodicTask(pObject,pFunc,nPeriod,nDelayedExecution);
    EOS_Scheduler.addPeriodicTask(pTask,nPriority);
}


//-------------------------------------------------------------------
// onDelay
//! adds a delayed task to the scheduler
//! \param pObject is the pointer to the object containing the member
//! function being executed
//! \param pFunc is the pointer to the member function
//! \param nDelayedExecution is the delay when the task shall be
//! executed
//! \param nPriority is the priority for execution (the higher the
//! earlier it will be called in case of conflicts), per default 0
//! \code
//! CMyClass::CMyClass()
//! {
//!     schedule(this,&CMyClass::myIdleFunction);
//! }
//! \endcode
template <class T_OBJECT>
void onDelay(T_OBJECT* pObject,
             void(T_OBJECT::* pFunc)(),
             unsigned int nDelayedExecution,
             uint8 nPriority = 0)
{
    CEOS_DelayedTask* pTask = new CEOS_DelayedTask(pObject,pFunc,nDelayedExecution);
    EOS_Scheduler.addDelayedTask(pTask,nPriority);
}


#endif
